@@ -48,12 +48,12 @@
 #' m2 = marker(trio, ch = 1:2, alleles = 1:2, afreq = c(0.5, 0.5))
 #' oneMarkerDistribution(trio, ids = c("fa", "mo"), partialmarker = m2)
 #'
-#' # A different example: The genotype distribution of an individual (id = 5)
+#' # A different example: The genotype distribution of an individual (id = 8)
 #' # whose half cousin (id = 9) is homozygous for a rare allele.
 #' y = halfCousinPed(degree = 1)
 #' snp = marker(y, `9` = "a", alleles = c("a", "b"), afreq = c(0.01, 0.99))
 #' plot(y, snp)
-#' oneMarkerDistribution(y, ids = 5, partialmarker = snp)
+#' oneMarkerDistribution(y, ids = 8, partialmarker = snp)
 #'
 #' @export
 oneMarkerDistribution = function(x, ids, partialmarker, loop_breakers = NULL,
@@ -78,14 +78,10 @@ oneMarkerDistribution = function(x, ids, partialmarker, loop_breakers = NULL,
   onX = isXmarker(m)
 
   if (verbose) {
-    cat(sprintf("Partial marker (%s):\n", ifelse(onX, "X-linked", "autosomal")))
+    cat("Known genotypes:\n")
     print(m)
-    cat("==============================\n")
-    msg = "Computing the %sgenotype probability distribution for individual%s: %s\n"
-    if(length(ids) == 1)
-      cat(sprintf(msg, "", "", ids))
-    else
-      cat(sprintf(msg, "joint ", "s", toString(ids)))
+    cat("\nChromosome type    :", ifelse(onX, "X-linked", "autosomal"))
+    cat("\nTarget individuals :", toString(ids), "\n")
   }
 
   starttime = Sys.time()
@@ -127,10 +123,15 @@ oneMarkerDistribution = function(x, ids, partialmarker, loop_breakers = NULL,
   }
 
   # Compute marginal
-  marginal = likelihood(x, marker1 = m, eliminate = eliminate)
+  marginal = likelihood(x, markers = m, eliminate = eliminate)
   if (marginal == 0)
       stop2("Partial marker is impossible")
 
+  if(verbose) {
+    cat("Marginal likelihood:", marginal, "\n")
+    cat("Calculations needed:", nrow(grid.subset), "\n")
+  }
+
   # Create list of all markers
   mlist = lapply(1:nrow(grid.subset), function(i) {
     r = grid.subset[i,]
@@ -139,10 +140,10 @@ oneMarkerDistribution = function(x, ids, partialmarker, loop_breakers = NULL,
   # Calculate likelihoods and insert in result array
   probs[probs.subset] = likelihood(x, mlist, eliminate = eliminate)
 
-  res = probs/marginal
-  if (verbose) {
-    cat("\nAnalysis finished in", round(Sys.time() - starttime, 2), " seconds\n")
-  }
+  # Timing
+  totalTime = format(Sys.time() - starttime, digits = 3)
+  if(verbose)
+    cat("\nAnalysis finished in", totalTime, "\n")
 
-  res
+  probs/marginal
 }
